//file name converter and file management functions
//Luke Lenhart (2007)
//See /docs/License.txt for details on how this code may be used.

#include "../File.h"
#include "../DebugRouter.h"
#include "evil_windows.h"
#include <shlobj.h>

namespace
{
    std::vector<std::string> FindFilesOfType(const std::string &path, bool getDirs)
    {
        std::string findStr=path.empty()?"*.*":path+"\\*.*";

        //start the find process
        WIN32_FIND_DATA file;
        HANDLE cur=FindFirstFile(findStr.c_str(),&file);
        if (cur==INVALID_HANDLE_VALUE || cur==0)
            return std::vector<std::string>();

        //process the files
        std::vector<std::string> list;
        do
        {
            bool isDir=(file.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)!=0;
            if (getDirs==isDir && std::string(file.cFileName)!="." && std::string(file.cFileName)!="..")
            {
                list.push_back(file.cFileName);
            }
        } while (0!=FindNextFile(cur, &file));

        FindClose(cur);

        return list;
    }
}

namespace MPMA
{
    //Construct this with a non-absolute filename using either windows or linux style, to
    // generate a filename for whatever platform this is running on.
    //Filenames that begin with ~ are relative to the users home directory.
    Filename::Filename(const std::string &filename)
    {
        if (filename.empty()) return;
        
        //
        uint len=filename.size();
        fname.reserve(len);
        uint startChar=0;
        if (filename[0]=='/' || filename[0]=='\\') startChar=1;
        
        if (filename.size()>startChar && filename[startChar]=='~') //home directory
        {
            ++startChar;
            
            //set string to absolute path of the users home directory
            char path[MAX_PATH];
            if(FAILED(SHGetFolderPath(0, CSIDL_PERSONAL, 0, 0, path)))
                fname="."; //better than nothing?
            else
                fname=path;
        }
        
        for (uint i=startChar; i<len; ++i)
        {
            char c=filename[i];
            //if (c==':') continue; //some platforms totally choke on this char, so totally remove it to maintain consistancy
            if (c=='/') c='\\';
                fname.push_back(c);
        }
        
        PurifyFilename();
    }

    // --

    //create a directory if it does not already exist
    bool Directory::CreateDirectory()
    {
        //make parents of that dir if we need to
        Directory od(*this);
        if (od.path.length()!=0)
        {
            od.StepOut();
            if (od.path.length()!=0)
            {
                if (!od.CreateDirectory())
                    return false;
            }
        }
        
        //do the final make
        if (!CreateDirectoryA(path.c_str(), 0))
        {
            int err=GetLastError();
            if (err!=ERROR_ALREADY_EXISTS)
            {
                ErrorReport()<<"Create directory failed on \""<<path<<"\".\n";
                return false;
            }
        }
        
        return true;
    }

    //retrieves a list of all files in this directory
    std::vector<std::string> Directory::GetFiles() const
    {
        return FindFilesOfType(path, false);
    }
    
    //retrieves a list of all directories in this directory (no . or ..)
    std::vector<std::string> Directory::GetSubDirectories() const
    {
        return FindFilesOfType(path, true);
    }
}
