//Luke Lenhart, 2007
//(filename is different to work around a msvc ide bug that prevented compilation)
//See /docs/License.txt for details on how this code may be used.

#include "../Locks.h"
#include "../MemMan.h"
#include "evil_windows.h"

// -- mutex setup

namespace MPMA
{

MutexLock::MutexLock()
{
    if (mMan.IsReady())
    {
        isMemManaged=true;
        crit=new2(CRITICAL_SECTION,CRITICAL_SECTION);
    }
    else
    {
        isMemManaged=false;
        crit=new CRITICAL_SECTION;;
    }
    InitializeCriticalSection((CRITICAL_SECTION*)crit);
}

MutexLock::~MutexLock()
{
    DeleteCriticalSection((CRITICAL_SECTION*)crit);
    CRITICAL_SECTION* theCrit=(CRITICAL_SECTION*)crit;
    if (isMemManaged)
        delete2(theCrit);
    else
        delete theCrit;
    crit=0;
}


// -- mutex use

TakeMutexLock::TakeMutexLock(MutexLock &critSection):
    crit(critSection)
{
    EnterCriticalSection((CRITICAL_SECTION*)crit.crit);
}

TakeMutexLock::~TakeMutexLock()
{
    LeaveCriticalSection((CRITICAL_SECTION*)crit.crit);
}


// -- BlockingObject

struct BlockObjData
{
    SpinLock lock;
    HANDLE handEvent;

    void SignalAll()
    {
        SetEvent(handEvent);
    }
};

BlockingObject::BlockingObject()
{
    isSet=false;
    clearCount=0;

    pdata=new2(BlockObjData,BlockObjData);
    BlockObjData *bodata=(BlockObjData*)pdata;

    bodata->handEvent=CreateEvent(0, true, true, 0);
}

BlockingObject::~BlockingObject()
{
    BlockObjData *bodata=(BlockObjData*)pdata;
    
    CloseHandle(bodata->handEvent);

    delete2(bodata);
}

//Marks the object as cleared, so nothing is blocking on it.
void BlockingObject::Clear()
{
    BlockObjData *bodata=(BlockObjData*)pdata;
    
    TakeSpinLock takeLock(bodata->lock);
    ++clearCount;
    isSet=false;
    bodata->SignalAll();
}

//Marks the object as set, so any all Waits will block on it.
void BlockingObject::Set()
{
    BlockObjData *bodata=(BlockObjData*)pdata;

    TakeSpinLock takeLock(bodata->lock);
    isSet=true;

    ResetEvent(bodata->handEvent);
}

//Blocks and waits for as long as the object is set (or if time is non-max, then until that many milliseconds pass).  Returns false only on timeout.
bool BlockingObject::WaitUntilClear(bool setOnReturn, uint timeToWait)
{
    BlockObjData *bodata=(BlockObjData*)pdata;

    TakeSpinLock takeLock(bodata->lock);
    if (!isSet)
    {
        clearCount=0;
        return true;
    }
    else if (clearCount>0)
    {
        --clearCount;
        return true;
    }
    
    bool retval=true;

    takeLock.Leave();
    if (timeToWait==0xffffffff)
    {
        WaitForSingleObject(bodata->handEvent, INFINITE);
    }
    else
    {
        uint ret=WaitForSingleObject(bodata->handEvent, timeToWait);
        retval=ret!=WAIT_TIMEOUT;
    }

    takeLock.Take();
    if (setOnReturn)
    {
        isSet=true;
        ResetEvent(bodata->handEvent);
    }
    --clearCount;

    return retval;
}

};
