//derived geometric objects
//Luke Lenhart (2005)
//See /docs/License.txt for details on how this code may be used.

#pragma once

#include "Geo.h"

namespace GEO
{
//3d line
struct Line
{
    Vector3 pos; //position of a point on the line
    Vector3 dir; //direction of the line

    //
    inline Line() {}
    inline Line(const Vector3 &inPos, const Vector3 &inDir)
        { pos=inPos; dir=inDir; }
};

//3d plane
struct Plane
{
    float c0,c1,c2,c3; //plane coeffecients

    //
    inline Plane()
        { c0=c1=c2=c3=0.0f; }
    //from a position and a normal
    inline Plane(const Vector3 &inPos, const Vector3 &inNormal)
    {
        Normal()=inNormal;
        Renormalize();
        c3=-VecDot(inPos,Normal());
    }
    //from the 4 coeffecients
    inline Plane(float inC0,float inC1,float inC2,float inC3): c0(inC0), c1(inC1), c2(inC2), c3(inC3)
        { Renormalize(); }

    //gets normal of plane
    inline Vector3& Normal() const { return *((Vector3*)this); }

    //normalizes normal of plane
    inline void Renormalize() { Normal().Normalize(); }
};

}; //namespace GEO
