//!\file UDP.h UDP packet flinging
//written by Luke Lenhart, 2008
//See /docs/License.txt for details on how this code may be used.

#pragma once

#include "Common.h"
#include <vector>

class MemMan; //FD

namespace NET
{
    //!Used to recieve udp packets on a single port.
    class UDPReceiver
    {
    public:
        //!Binds a socket to a port in order to receive upd packets.  If the port cannot be bound or an error occurs, returns 0.
        static UDPReceiver* CreateReceiver(uint16 port);

        //!Unbinds and frees a UDPReceiver.
        static void FreeReceiver(UDPReceiver *receiver);

        //!Receives the next packet and appends it to data.  source (if not 0) will be filled with the address the packet came from.  Returns false if there was nothing to receive.
        bool Receive(std::vector<uint8> &data, Address *source=0);

        //!Retrieves the port to which the receiver is bound
        inline uint16 GetPort() const { return port; }

        //TODO: Callback mechanism to notify when data is recieved so they don't have to poll.

    private:
        //we are the only allocator
        UDPReceiver();
        inline ~UDPReceiver() {}
        inline UDPReceiver(const UDPReceiver&) {}
        inline void operator=(const UDPReceiver&) {}
        friend class ::MemMan;

        //
        int sock;
        uint16 port;
    };

    //!Used to send udp packets.
    class UDPSender
    {
    public:
        //!Creates a socket to be used to send upd packet.  Returns 0 on failure.
        static UDPSender* CreateSender();

        //!Frees a UDPSender.
        static void FreeSender(UDPSender *sender);

        //!Sends a packet of data.  Returns false if the send failed.
        bool Send(const Address &addr, const void *data, uint dataLen);

    private:
        //we are the only allocator
        UDPSender();
        inline ~UDPSender() {}
        inline UDPSender(const UDPSender&) {}
        inline void operator=(const UDPSender&) {}
        friend class ::MemMan;

        //
        int sock;
    };
}
