//provides helpers for platform-specific socket operations.
//this is meant for internal framework use only.
//Luke Lenhart, 2008
//See /docs/License.txt for details on how this code may be used.

#include "../PlatformSockets.h"
#include "../../base/Vary.h"
#include <errno.h>
#include <fcntl.h>

namespace NET
{
    //sets a socket to be blocking or nonblocking
    bool Internal_SetSocketBlocking(int sock, bool block)
    {
        int flags=fcntl(sock, F_GETFL, 0);
        if (block) flags&=~O_NONBLOCK;
        else flags|=O_NONBLOCK;

        if (fcntl(sock, F_SETFL, flags)<0)
            return false;
        return true;
    }

    //gets a string that represents a socket error for the most recent operation
    std::string Internal_GetLastError()
    {
        MPMA::Vary reason;
        if (errno==EADDRINUSE) reason="Address already in use";
        else if (errno==EADDRNOTAVAIL) reason="Address not available";
        else if (errno==EACCES) reason="Access denied";
        else if (errno==ETIMEDOUT) reason="Connection timed out";
        else if (errno==ECONNREFUSED) reason="Connection refused";
        else if (errno==ENETUNREACH) reason="Destination unreachable";
        else if (errno==EAFNOSUPPORT) reason="Not supported";
        else if (errno==EINVAL) reason="Invalid argument";
        else
        {
            reason="Unknown(";
            reason+=errno;
            reason+=")";
        }
        return reason;
    }

    //closes a socket
    void Internal_CloseSocket(int sock)
    {
        close(sock);
    }

    //returns whether the error from the most recent socket operations indicates a loss of connection, or whether the socket was nicely closed
    bool Internal_IsLastSocketErrorADisconnect()
    {
        if (errno==ECONNRESET || errno==ENOTCONN || errno==ETIMEDOUT || errno==EBADF || errno==ENOTSOCK)
            return true;
        else
            return false;
    }
}
