//provides helpers for platform-specific socket operations.
//this is meant for internal framework use only.
//Luke Lenhart, 2008
//See /docs/License.txt for details on how this code may be used.

#include "../PlatformSockets.h"
#include "../../base/Vary.h"

namespace NET
{
    //sets a socket to be blocking or nonblocking
    bool Internal_SetSocketBlocking(int sock, bool block)
    {
        unsigned long val=(block?0:1);
        if (ioctlsocket(sock, FIONBIO, &val)==SOCKET_ERROR)
            return false;
        return true;
    }

    //gets a string that represents a socket error for the most recent operation
    std::string Internal_GetLastError()
    {
        int errnum=WSAGetLastError();
        MPMA::Vary reason;
        if (errnum==WSAENETDOWN) reason="Network failure";
        else if (errnum==WSAEADDRINUSE) reason="Address already in use";
        else if (errnum==WSAEADDRNOTAVAIL) reason="Address not available";
        else if (errnum==WSAETIMEDOUT) reason="Connection timed out";
        else if (errnum==WSAECONNREFUSED) reason="Connection refused";
        else if (errnum==WSAENETUNREACH) reason="Destination unreachable";
        else if (errnum==WSAEINVAL) reason="Invalid argument";
        else
        {
            reason="Unknown(";
            reason+=errnum;
            reason+=")";
        }
        return (const std::string&)reason;
    }

    //closes a socket
    void Internal_CloseSocket(int sock)
    {
        closesocket(sock);
    }

    //returns whether the error from the most recent socket operations indicates a loss of connection, or whether the socket was nicely closed
    bool Internal_IsLastSocketErrorADisconnect()
    {
        int errnum=WSAGetLastError();
        if (errnum==WSAENETDOWN || errnum==WSAENOTCONN || errnum==WSAENETRESET || errnum==WSAESHUTDOWN || errnum==WSAECONNABORTED || errnum==WSAETIMEDOUT || errnum==WSAECONNRESET || errnum==WSAENOTSOCK || errnum==WSAEINVAL)
            return true;
        else
            return false;
    }
}
